/*
    This file is part of ydotool.
    Copyright (C) 2018-2019 ReimuNotMoe

    This program is free software: you can redistribute it and/or modify
    it under the terms of the MIT License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#ifndef YDOTOOL_COMMONINCLUDES_HPP
#define YDOTOOL_COMMONINCLUDES_HPP


#include <iostream>
#include <algorithm>
#include <functional>
#include <sstream>
#include <stdexcept>
#include <thread>
#include <deque>

#include <cstdio>
#include <cstdlib>
#include <cerrno>
#include <ctime>

#include <fcntl.h>
#include <dirent.h>
#include <dlfcn.h>
#include <signal.h>

#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/un.h>

#include <sys/epoll.h>

#include <uInputPlus/uInput.hpp>
#include <evdevPlus/evdevPlus.hpp>

#include <boost/program_options.hpp>
#include <boost/crc.hpp>

#endif //YDOTOOL_COMMONINCLUDES_HPP
